'
' Sequentailly read BMP files from an SD Card formatted as FAT16
' Display the bitmap on the Amicus18 Colour graphic LCD shield
'
' Written by Les Johnson. September 2011
'
    Declare Optimiser_Level = 3                 ' Maximum optimisation
'
' SD Card Declarations
' Default settings for the Amicus Touch Screen Colour LCD Shield
'
    $define SD_SDO PORTC.5                      ' SPI DO to SD card DI pin
    $define SD_SDI PORTC.4                      ' SPI DI to SD card DO pin
    $define SD_CLK PORTC.3                      ' SPI CLK to SD card CLK pin
    $define SD_CS PORTA.4                       ' SPI CS to SD card CS pin (active low)
    $define SD_Power PORTA.5                    ' Enable/Disable the SD card's VDD (active low)
    $define SD_Detect PORTA.2                   ' Detect if the SD card is inserted
   
    Include "ILI9320.inc"                       ' Load the ILI9320 graphic LCD routines into the program
    Include "FAT16.inc"                         ' Load the FAT16 reading routines into the program

    Dim bLoop As Byte                           ' Used to read and discard the bmp file's header
    Dim BMP_Xpos As Byte                        ' LCD's pixel X position
    Dim BMP_Ypos As Word                        ' LCD's pixel Y position
    
    Dim AmountOfFiles As Word                   ' Holds the amount of files found on the SD card
    Dim FileSelect As Word                      ' Holds the file number to select for displaying
    Dim FileNowDisplaying As Word               ' Holds the current file number that is being displayed

'----------------------------------------------------------------------------------------------------------
    GoTo Main                                    ' Jump over the subroutines
'----------------------------------------------------------------------------------------------------------
' Display an image on the graphic LCD
' Input     : None
' Output    : None
' Notes     : The file containing the image must already have been opened
'           : The .bmp file must be format 16-bit RGB565 without a colour lookup table
'
Display_Image:
'
' Discard the BMP header information
'
    bLoop = 0
    Repeat
        File_ReadWord()
        Inc bLoop
    Until bLoop >= $23
'
' Read and display the bmp file
'                                       
    Glcd_WriteReg($60,%1010011100000000)                    ' Reverse the direction of the LCD's scan, otherwise images are mirrored 
    Glcd_SetWindow(0, 0, 239, 319)                          ' Create a window the full size of the LCD's screen
    Clear Glcd_CS                                           ' Enable the LCD
    hSetCGRAM(0,0)                                          ' Move to location 0,0 within CGRAM
    While 1 = 1                                             ' Create an infinite loop
        wPixelColour = File_ReadWord()                      ' Read 16-bits from the file and use it as the pixel's colour
        hWriteDataWord(wPixelColour)                        ' Write the 16-bits directly to the LCD
        If FAT_bResponse = cFAT_EndOfFile Then Break        ' Exit the loop when the End-Of-File is reached
    Wend                                                    ' Close the loop
    Set Glcd_CS                                             ' Disable the LCD
    DelayMS 4096                                            ' Keep the image on the LCD for a while
    Return
'----------------------------------------------------------------------------------------------------------
' The main program loop starts here
' Open a sequence of .bmp files and display them on the Amicus18 Graphic LCD shield
'
Main:
    Glcd_Cls(clBlack)                                           ' Clear the LCD with the colour black

    If FAT_Init() = True Then                                   ' Did the media initialise?
        '
        ' Count the amount of valid files in the directory
        '    
        AmountOfFiles = 0
        Repeat          
            Dir_List()                                          ' Read a valid file
            If bListResponse = 1 Then Inc AmountOfFiles         ' Increment the Amount of Files if a valid file found
        Until bListResponse = 0                                 ' Until the end of the directory is found 
        
        FileNowDisplaying = 0                                   ' Reset the File selection variable
        While 1 = 1                                             ' Yes. So create an infinite loop
            FileSelect = 0                                      ' Reset the File selection variable
            Repeat                                              ' Form a loop to read the directory
                Dir_List()                                      ' Read a single valid file
                If bListResponse = 1 Then                       ' Have we reached a valid file?
                    If FileSelect = FileNowDisplaying Then      ' Yes. So is it the next file in the directory?
                        tStartedListingDirectory = False        ' Yes. So reset the Directory Listing subroutine
                        Break                                   ' And exit the loop
                    EndIf
                    Inc FileSelect                              ' Increment the selected file variable
                EndIf
            Until bListResponse = 0                             ' Until the end of the directory is found 
            FAT_Filename = FAT_sInternalFilename                ' Transfer the filename returned from Dir_List() into FAT_Filename        
            GoSub FAT_OpenFile_Raw                              ' Open a file based upon the raw filename held in FAT_Filename
            GoSub Display_Image                                 ' Display the the image on the LCD
            Inc FileNowDisplaying                               ' Increment the current file being played variable
            If FileNowDisplaying = AmountOfFiles Then           ' Have we reached the last file in the directory ?
                FileNowDisplaying = 0                           ' Yes. So start again at the beginning
            EndIf
        Wend
    Else
        HRSOut "FAT Init Fail\r"
    EndIf
    Stop


